#include "aaAlphabet.h"


AAAlphabet::AAAlphabet(){}

AAAlphabet::AAAlphabet(string fname){

    alph2indx = vector<int>(ALPHABET_MAX_SIZE);

    ifstream file(fname);
    if (file.is_open()){   //checking whether the file is open
        string str;
        int indx = 0;
        while (getline(file, str)) {
            compressedAlphabet.push_back(str);
            alphabet.push_back(str[0]);
        
            if(str.length() > 0){
                for(char c : str){
                    alphabetMapping[c] = str[0];
                    int alphi = c;
                    assert(alphi >= 0 && alphi < ALPHABET_MAX_SIZE);
                    alph2indx[alphi] = indx;
                }
            }
            indx++;
        }
        file.close(); //close the file object.
    } else{ 
        cerr << "SSTPred: I/O Error - Error reading '"<< fname <<"' file." << endl;
        exit(1);
    }
}

AAAlphabet::AAAlphabet(vector<string> inpalph){
    alph2indx = vector<int>(ALPHABET_MAX_SIZE, -1);

    compressedAlphabet = inpalph;
    int indx = 0;
    for(string resid : inpalph){
        alphabet.push_back(resid[0]);
        for(char c : resid){
            alphabetMapping[c] = resid[0];
            int alphi = c;
            assert(alphi >= 0 && alphi < ALPHABET_MAX_SIZE);
            alph2indx[alphi] = indx;
        }
        indx++;
    }
}

int AAAlphabet::operator[](char key){
    int indx = alphabetMapping[key];
    assert(indx >= 0 && indx < ALPHABET_MAX_SIZE);
    return alph2indx[indx];
}

int AAAlphabet::operator[](string key){
    int row = 0;
    for(int i = 0; i < key.length(); i++){
        row += (int) pow(alphabet.size(), i) * (int) (*this)[key[i]];
    }
    return row;
}

string AAAlphabet::operator()(string code, bool isAlignment = false){
    string resCode = "";
    if(isAlignment){
        for(char c : code){
            if(c == '-') resCode += c;
            else
                resCode += alphabetMapping[c];
        }
    } else {
        for(char c : code){
            resCode += alphabetMapping[c];
        }
    }
    
    return resCode;
}

char AAAlphabet::operator()(char code){
    return alphabetMapping[code];
}

string AAAlphabet::getAlphabet(){
    assert(alphabet.size() > 0);
    return string(alphabet.begin(), alphabet.end());
}

vector<string> AAAlphabet::getCompressedAlphabet(){
    assert(alphabet.size() > 0);
    assert(alphabet.size() == compressedAlphabet.size());
    return compressedAlphabet;
}

int AAAlphabet::size(){
    return alphabet.size();
}