#include "cmdLineParser.h"

int Parser::isOption(const char *arg){
	if(arg == NULL || strlen(arg) < 2 )
		return 0;

   	if(arg[0] == '-' && !isdigit(arg[1]))
      	return 1;
   	return 0;
}

void Parser::replaceString(string& subject, const string& search, const string& replace){
    size_t pos = 0;
    while ((pos = subject.find(search, pos)) != std::string::npos) {
         subject.replace(pos, search.length(), replace);
         pos += replace.length();
    }
}

void Parser::cmdLineHelp(){
	cerr << "SSTSUM Options: \n" ;
	for( int i = 0 ; i < helpOpt.size() ; i++){
		cerr.setf(ios::left);
        replaceString(helpOpt[i][2], "\n", "\n       ");
		cerr << "    " <<  helpOpt[i][0];
	    cerr << " " << helpOpt[i][1]  << "\n       ";
		cerr << helpOpt[i][2] << endl;
	}
}

int Parser::inputParser(char** optn, const string& argv){
    string errormsg = "SSTSUM: Unable to associate the argument \'" + argv + "\' with any valid options!\n Try \'SSTSUM --help\' for more information.\n";

    if(!strcmp(*optn, "-p")){
        inpfile = argv;
        return 1;
    }

    if(!strcmp(*optn, "-a")){
        sseFile = argv;
        return 1;
    }
    
    if(!strcmp(*optn, "-m")){
        initMFile = argv;
        return 1;
    } 

    if(!strcmp(*optn, "-d")){
        initDirFile = argv;
        return 1;
    } 

    if(!strcmp(*optn, "-o")){
        outfldr = argv;
        return 1;
    } 

    if(!strcmp(*optn, "-t")){
        try {
            timeUB = stoi(argv);
        }
        catch(const std::exception& e) {
            cerr << errormsg;
            cerr << e.what() << '\n';
            exit(0);
        }
        return 1; 
    }

    if(!strcmp(*optn, "-i")){
        try {
            inferencetype = stoi(argv);
        }
        catch(const std::exception& e) {
            cerr << errormsg;
            cerr << e.what() << '\n';
            exit(0);
        }
        return 1; 
    }
   
    return 0;
} 


Parser::Parser(){}

Parser::Parser(int argc, char **argv){
    bool isOptn = true;
    char ** optn;
    
    if(argc == 2 && !strcmp(argv[1],"--help")){
        cmdLineHelp();
        exit(0); 
    } 

    for(int i = 1; i < argc; i++){
        if(isOption(argv[i])){
            optn = &argv[i];
            isOptn = false;
        } else if(isOptn){
            cerr << "SSTSUM: Unable to associate the argument \'" << argv[i] << "\' with any valid options!\n";
            cerr << "Try \'SSTSUM --help\' for more information.\n";
            exit(0);
        } else {
            if(inputParser(optn, argv[i]))
                isOptn = true;
            else {
                cerr << "SSTSUM: Unable to associate the argument \'" << argv[i] << "\' with any valid options!\n";
                cerr << "Try \'SSTSUM --help\' for more information.\n";
                exit(0);
            }
        }
    } 

    if(!(inferencetype == 0 || inferencetype == 1)){
        cerr << "SSTSUM: Use 0 or 1 with -i option.\n";
        cerr << "Try \'sstmat --help\' for more information.\n";
        exit(0);
    }
}


