#include "dirManager.h"

DirManager::DirManager(){}

DirManager::DirManager(const int upperBound){
    alphaM = vector<Dirichlet>(upperBound+1);
    alphaI = vector<Dirichlet>(upperBound+1);
}

DirManager::DirManager(const string fname, int timeUB){
    load_dir_data(fname, timeUB);
}

void DirManager::load_dir_data(const string filename, int timeUB){

    ifstream infile(filename.c_str(),ios::in);
    assert(infile);
    
    char buff[10000];

    alphaM.push_back(Dirichlet());
    alphaI.push_back(Dirichlet());
    int k = 1;
   
    while (!infile.eof()) {

        infile.getline(buff,10000);
        stringstream ss;
        ss << buff;

        int stime;
        double ma1, ma2, ia1, ia2, ia3;

        while(ss >> stime >> ma1 >> ma2 >> ia1 >> ia2 >> ia3){ // rest of the columns
            
            Dirichlet m(2, {ma1, ma2}, (ma1 + ma2));
            Dirichlet i(3, {ia1, ia2, ia3}, (ia1 + ia2 + ia3));

            if(alphaM.size() == timeUB+1) break;

            while(k != stime){
                alphaM.push_back(alphaM[alphaM.size() - 1]);
                alphaI.push_back(alphaI[alphaI.size() - 1]);
                k++;
            } 

            alphaM.push_back(m);
            alphaI.push_back(i);

            assert(stime+1 == (alphaM.size()));
            assert(stime+1 == (alphaI.size()));
            
            k++;

        }
    }
    while(alphaM.size() < timeUB+1){
        alphaM.push_back(alphaM[alphaM.size() - 1]);
        alphaI.push_back(alphaI[alphaI.size() - 1]);
        k++;
    } 

    assert(alphaM.size() == timeUB+1);
    assert(alphaI.size() == timeUB+1);

    infile.close();
}

Dirichlet DirManager::getMatchDirichlet(const int n){
    assert(n >= 0 && n < alphaM.size());
    assert(alphaM[n].getK() == 2);
    return alphaM[n];
}

Dirichlet DirManager::getIndelDirichlet(const int n){
    assert(n >= 0 && n < alphaI.size());
    assert(alphaI[n].getK() == 3);
    return alphaI[n];
}

int DirManager::getSize(){
    assert(alphaM.size() == alphaI.size());
   
    return alphaM.size();
}

void DirManager::insert(Dirichlet& dir, int indx){
    int k = dir.getK();
    if(k == 2){
        assert(indx >= 0 && indx < alphaM.size());
        alphaM[indx] = dir;
    } else if(k == 3){
        assert(indx >= 0 && indx < alphaI.size());
        alphaI[indx] = dir;
    } else assert(false);
}

void DirManager::print(string fn){

    assert(alphaM.size() == alphaI.size());
    
    ofstream out(fn,ios::out);
    if(!out) cerr << "SSTSUM: Invalid file " << fn << endl;
    assert(out);

    for(int i = 0; i < alphaM.size(); i++){
        Dirichlet dirm = alphaM[i];
        Dirichlet diri = alphaI[i];

        stringstream ss;

        if(dirm.isEmpty() && diri.isEmpty())
            continue;
        
        ss << i << " ";
        vector<double> alphm = dirm.getAlpha();
        assert(alphm.size() == 2);
        for(double val : alphm){
            ss << setprecision(15) << val << " ";
        }
        ss << "\t";
        vector<double> alphi = diri.getAlpha();
        assert(alphi.size() == 3);
        for(double val : alphi){
            ss << setprecision(15) << val << " ";
        } ss << endl;

        cout << ss.str();
        out << ss.str();
    }

    out.close();
}
