#include "emDir.h"

EMDir::EMDir(){
}


EMDir::EMDir(const string dirFile, const string alignfile, AAAlphabet& alph, int timeUB){
        
    ssealph = alph; 

    dirmg = DirManager(dirFile, timeUB);
    assert(dirmg.getSize() == timeUB+1);
    inferreddm = DirManager(dirmg.getSize());

    assert(dirmg.getSize() == timeUB+1);
    binnedAlignments = parseBinnedAlignmentData(alignfile, ssealph, timeUB);
}

void EMDir::anneal(){

    string tempParams = 
        "\n========================================= Parameters =========================================\n"
        "Temperature                          = " + num2str(startingKelv, 0, 10) + "\n"
        "Cooling Temperature Probability      = " + num2str(kelvProb, 2, 10) + "\n"
        "Stopping Temperature                 = " + num2str(stoppingKelv, 4, 10) + "\n"
        "Number of Perturbations              = " + num2str(perturbations, 0, 10) + "\n"
        "Tolerance                            = " + num2str(tolerance, 4, 10) + "\n"
        "==============================================================================================\n";
    cout << tempParams;
    
    vector<int> (SSEAlign::*pt2TransitionVecFunc)() = NULL; 
    
    for(int n = 1; n < dirmg.getSize(); n++){
    
        assert(n < binnedAlignments.size());
        if(binnedAlignments[n].size() < 1) continue;

        Dirichlet dirM = dirmg.getMatchDirichlet(n);
        pt2TransitionVecFunc = &SSEAlign::getMTransitionVec;
        Dirichlet inferreddirm = annealDirichlet(n, dirM, pt2TransitionVecFunc);
        inferreddm.insert(inferreddirm, n);

        Dirichlet dirI = dirmg.getIndelDirichlet(n);
        pt2TransitionVecFunc = &SSEAlign::getITransitionVec;
        Dirichlet inferreddiri = annealDirichlet(n, dirI, pt2TransitionVecFunc);
        inferreddm.insert(inferreddiri, n);
    }

}

Dirichlet EMDir::annealDirichlet(int binid, Dirichlet& initdir, vector<int> (SSEAlign::*dirTransitionFunction)()){

    string tempParams = 
        "\n========================================= Parameters =========================================\n"
        "Temperature                          = " + num2str(startingKelv, 0, 10) + "\n"
        "Cooling Temperature Probability      = " + num2str(kelvProb, 2, 10) + "\n"
        "Stopping Temperature                 = " + num2str(stoppingKelv, 6, 10) + "\n"
        "Number of Perturbations              = " + num2str(perturbations, 0, 10) + "\n"
        "Tolerance                            = " + num2str(tolerance, 6, 10) + "\n"
        "Bin ID                               = " + num2str(binid, 0, 10) + "\n"
        "==============================================================================================\n";
    cout << tempParams;

    double acceptedCost, startingCost = 0;

    startingCost = cost_objective_function(initdir, binid, dirTransitionFunction);
    
    int nIgnored = 0, j = 0;
    kelvin = startingKelv;
    acceptedCost = startingCost;
    Dirichlet acceptedDir = initdir;
  
    string criteria;
    string iterLegend;
    string dashedline = string(94, '-');

    while(kelvin >= stoppingKelv){

        for(j = iter; j < perturbations; j++){  

            Dirichlet perturbedDir = perturbDirichlet(acceptedDir);
            
            if(!perturbedDir.isValid()){
                nIgnored++;
                continue;
            }

            double newCost = cost_objective_function(perturbedDir, binid, dirTransitionFunction);
            double delta_obj = newCost - acceptedCost;

            iterLegend = "Accepted Dirichlet  : \n" + acceptedDir.print() + "\n"; 
            iterLegend += "Perturbed Dirichlet : \n" + perturbedDir.print() + "\n"; 

            iterLegend += 
                num2str(kelvin, 0, 8) + "\t" + to_string(j) + "\t" + num2str(nits2bits(newCost), 2, 14) + "\t" + 
                num2str(nits2bits(acceptedCost), 2, 14) + "\t" + num2str(nits2bits(-delta_obj), 2, 10) + "\t";

            if(delta_obj > -tolerance && delta_obj < tolerance) delta_obj = 0;     
            
            if(delta_obj < 0){
                acceptedCost = newCost;
                acceptedDir = perturbedDir;
               
                criteria = "Accepted\n";

            } else {
                double prob = exp(-delta_obj/kelvin);
           
                double c = ((double) rand() / (RAND_MAX));
                assert(prob >= 0 && prob <= 1);
                assert(c >= 0 && c <= 1);
                
                if(prob > c){
                    acceptedCost = newCost;
                    acceptedDir = perturbedDir;
                  
                    criteria = "Accepted with prob. " + num2str(prob, 5, 6) + "\n";
                
                } else {
                    criteria = "Rejected\n";
                }
            }
        }
        kelvin = kelvin * kelvProb;
        iter = 0; 
        if(kelvin <= 10) perturbations = 20000;
    }
    cout << getLegend(acceptedCost, startingCost, j, binid, acceptedDir) << endl;

    return acceptedDir;
}

Dirichlet EMDir::perturbDirichlet(Dirichlet& dir){
    
    int randNum = rand() % 2; 
    assert(randNum == 0 || randNum == 1);

    vector<double> meanvec = dir.getMean();
    vector<double> perturbMean = meanvec;

    double dirkappa = dir.getKappa();
    double perturbKappa = dirkappa;

    // perturb kappa
    if(randNum){
        double delta = uniform_random(0.1, 10);
        randNum = rand() % 2; 
        assert(randNum == 0 || randNum == 1);
        perturbKappa = (randNum == 0) ? (dirkappa + delta) : (dirkappa - delta);

    // perturb mean    
    } else {
        perturbMean = (dir.getK() == 3) ? perturbProbVec(meanvec, 1000) : perturbProbVec(meanvec, 10000);
    }

    vector<double> perturbAlphaVec;
    double alphSum = 0;
    for(double m : perturbMean){
        double val = m*perturbKappa;
        perturbAlphaVec.push_back(val);
        alphSum += val;
    }

    //assert(perturbKappa > 0);
    Dirichlet perturbeddir(dir.getK(), perturbAlphaVec, alphSum); 

    return perturbeddir;
}

double EMDir::cost_objective_function(Dirichlet& dir, int binid, vector<int> (SSEAlign::*dirTransitionFunction)()){

    double objectiveCost = 0, dirCost = 0;;
  
    // Equation 5.3 - I(alpha)
    objectiveCost += dir.I_alpha(binnedAlignments[binid].size());

    //search for the optimal time for data
    for(int i = 0; i < binnedAlignments[binid].size(); i++){

        // Equation 5.3 - I(theta_i | alpha_i, t_i) + I(A_i | theta_i, t_i)
        vector<int> transitions = (binnedAlignments[binid][i].*dirTransitionFunction)();
        dirCost = dir.I_theta_plus_I_A(binnedAlignments[binid][i], transitions);
       
        objectiveCost += dirCost;
    }

    // Equation 5.4 2nd term of I(theta|alpha) + Equation 5.5 last term of I(A)
    objectiveCost += dir.constTerms(binnedAlignments[binid].size());
 
    return objectiveCost;
}

string EMDir::getLegend(double cost, double startingCost, int iter, int binid, Dirichlet& dir){
    string legend =  
        "========================================= SA Legend ==========================================\n"
        "Current cost                          : " + num2str(nits2bits(cost), 9, 10) + " bits\n" +
        "Initial cost                          : " + num2str(nits2bits(startingCost), 9, 10) + " bits\n" +
        "Bin ID                                : " + num2str(binid, 0, 10) + "\n" +
        "----------------------------------------------------------------------------------------------\n" +
        "Accepted Dirichlet : \n" +
        dir.print() + "\n"
        "==============================================================================================\n";
      
    return legend;    
}

double EMDir::uniform_random(double lower, double upper){
  	uniform_real_distribution<double> uni_dist(lower, upper);
	return uni_dist(random_number_generator);
}

// Generates a random number from Gamma distribution with parameters alpha and beta
double EMDir::gamma_random(double alpha, double beta){
  	gamma_distribution<double> gamma_dist(alpha, beta);
	return gamma_dist(random_number_generator);
}

vector<double> EMDir::perturbProbVec(const vector<double>& currentVec, const double& kappa){
    vector<double> perturbedVec(currentVec.size());

    double sum = 0;
    for(int i = 0; i < perturbedVec.size(); i++){

        perturbedVec[i] = gamma_random(currentVec[i] * kappa, 1.0);
        assert(perturbedVec[i] >= 0);
        sum += perturbedVec[i];
    }

    for(int i = 0; i < perturbedVec.size(); i++){
        perturbedVec[i] /= sum;
        assert(perturbedVec[i] >= 0);
    }

    return perturbedVec;
}

void EMDir::writeOutput(string outfldr){
    inferreddm.print(outfldr + "/inferred_dirichlet_" + currentDateTime() + ".txt");
}
