#include "sseAlignment.h"

SSEAlign::SSEAlign(){}

SSEAlign::SSEAlign(string hdr, string al1, string al2){

    header = hdr;
    align1 = al1;
    align2 = al2;

    assert(align1.length() == align2.length());
    assert(align1.length() > 6);
    
    matchRegion1 = "", matchRegion2 = "";
    insertRegion = "", deleteRegion = "";
    fsastr = "";

    char prev, curr;
    transitions = vector<vector<int>>(3, vector<int>(3, 0));
    totTrans = 0;

    for(int j = 0; j < align1.length(); j++){
        if(align1[j] != '-' && align2[j] != '-'){
            matchRegion1 += align1[j];
            matchRegion2 += align2[j];
            curr = 'M';
            string key = string(1, align1[j]) + align2[j];
            if (matchCounts.count(key))
                matchCounts[key] += 1;
            else 
                matchCounts[key] = 1; 

        } else if(align1[j] == '-'){
            deleteRegion += align2[j];
            curr = 'D';
            char key = align2[j];
            if (indelCounts.count(key))
                indelCounts[key] += 1;
            else 
                indelCounts[key] = 1; 

        } else if(align2[j] == '-'){
            insertRegion += align1[j];
            curr = 'I';
            char key = align1[j];
            if (indelCounts.count(key))
                indelCounts[key] += 1;
            else 
                indelCounts[key] = 1; 

        } else assert(false);

        if(j > 0){
            transitions[state2Indx(prev)][state2Indx(curr)] += 1;
            totTrans += 1;
        }
        fsastr += curr;
        prev = curr;
    }

    assert(align1.length() == (matchRegion1.length() + insertRegion.length() + deleteRegion.length()));
    assert(matchRegion1.length() == matchRegion2.length());
    
    int trsum = 0;
    for(int i = 0 ; i < transitions.size(); i++)
        trsum += accumulate(transitions[i].begin(), transitions[i].end(), 0);
    assert(trsum == totTrans);    

    mTransitions = {getTransitions('M', 'M'), 
                    getTransitions('M', 'I') + getTransitions('M', 'D')};

    iTransitions = {getTransitions('I', 'I') + getTransitions('D', 'D'), 
                    getTransitions('I', 'M') + getTransitions('D', 'M'),
                    getTransitions('I', 'D') + getTransitions('D', 'I')};                
}

int SSEAlign::state2Indx(char state){
    int indx = (int) (state - 'A') / 3;
    assert(indx == 1 || indx == 2 || indx == 4);

    return (int) indx/2;
}

vector<string> SSEAlign::getAlignments(){
    assert(align1.length() == align2.length());
    return {align1, align2};
}

vector<string> SSEAlign::getMatchRegion(){
    assert(matchRegion1.length() == matchRegion2.length());
    return {matchRegion1, matchRegion2};
}

string SSEAlign::getInsertRegion(){
    return insertRegion;
}

string SSEAlign::getDeleteRegion(){
    return deleteRegion;
}

string SSEAlign::getHeader(){
    return header;
}

string SSEAlign::getFSAstr(){
    return fsastr;
}

string SSEAlign::getFirstAlignment(){
    return align1;
}

string SSEAlign::getSecondAlignment(){
    return align2;
}

int SSEAlign::size(){
    return align1.length();
}

int SSEAlign::getTransitions(char type){
    return accumulate(transitions[state2Indx(type)].begin(), 
        transitions[state2Indx(type)].end(), 0);
}

int SSEAlign::getTransitions(char from, char to){
    return transitions[state2Indx(from)][state2Indx(to)];
}

vector<int> SSEAlign::getMTransitionVec(){
    return mTransitions;
}

vector<int> SSEAlign::getITransitionVec(){
    return iTransitions;
}

map<string, int> SSEAlign::getMatchCounts(){
    return matchCounts;
}


map<char, int> SSEAlign::getIndelCounts(){
    return indelCounts;
}
