#ifndef Submat_H
#define Submat_H

#include <iostream>
#include <iomanip>
#include <fstream>
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <cassert>
#include <cmath>
#include <string>
#include <stdint.h>
#include <map>
#include <vector>
#include <sstream>
#include <time.h>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_math.h>
#include <gsl/gsl_eigen.h>
#include <gsl/gsl_linalg.h>
#include <gsl/gsl_blas.h>
#include <gsl/gsl_sf.h> // special functions - include polygamma
#include "aaAlphabet.h"
#include "util.h"
using namespace std;

class Submat {
    private:
        AAAlphabet alphabet;
        vector<vector<double>> mat;
       
        vector<vector<double>>     S; // matrix of eigvectors of Mpow1
        vector<vector<double>>     D; // diagonal matrix of eigvalues of Mpow1
        vector<vector<double>>  invS; // inverse of S of Mpow1
        //vector<vector<vector<double>>> matMult;  

        bool valid = true; 
        int NMATRICES; 

        vector<double> stationarydist; // stationary distribution 
    
       /* Series of conditional MpowN matrices. These are precomputed by
        * the constructor, forall 1<=N<=1000. Each column of cMat[N] matrix 
        * adds up to 1. */
        vector<vector<vector<double>>> matSeries;                                                   
       /* Series of joint matrices. These are again precomputed by
        * the constructor along with the above (cMat) series (using 
        * the stationary distribution), forall 1<=N<=1000. 
        * Sum over the entire jMat[N] matrix adds up to 1. */
        vector<vector<vector<double>>> jointMSeries; 
        

        vector<vector<double>> readMatrix(string);
        vector<vector<double>> readBaseMatrix(string);
        void normalize(vector<vector<double>>&);
        void normalize(vector<double>&);
        bool eigenDecomposition();
        vector<vector<double>> get_matrix_inverse(vector<vector<double>>&);
        void initMatrix(vector<vector<double>>&, int);
        void printMatrix(vector<vector<double>>&);
        void printMatrix(vector<double>&);
  
        vector<vector<double>> exponentiate_matrix_efficiently(int n);
        void normalizeJoint(vector<vector<double>>&);
        void generateJointMatrix(int);
        void generateStationaryDistribution();
        void generateConditionalMatrices();
        void generateJointMatrices();
        void init();
    
    public:
        Submat();
        Submat(const string, int);
        Submat(vector<vector<double>>&, AAAlphabet&, int);
        Submat(vector<vector<double>>&, vector<vector<double>>&, vector<vector<double>>&, vector<vector<double>>&, vector<double>&, AAAlphabet&, int);
        Submat(AAAlphabet&, int);
        bool isValid();
        vector<vector<double>> compute_SubMat_N(int);

        double getCondlPr(char, char, int); // gets conditional pr(X|Y) for specified N
        double getJointPr(char, char, int); // gets joint pr(X,Y) for specified N
        double nullpr(char);
        AAAlphabet getAlphabet();
        vector<vector<double>> getBaseMat();
        vector<double> getStationaryDistribution();
        string printBaseMatrix();
        string printMatrix(string);
        double compute_I_M();
        void scaleMatrix();
        double getExpectedChange();
};
#endif
